@@ -16,13 +16,10 @@
 // under the License.
 
 use std::fmt::Debug;
-use std::io;
-use std::task::ready;
-use std::task::Context;
-use std::task::Poll;
+use std::sync::atomic::AtomicU64;
+use std::sync::atomic::Ordering;
 
-use async_trait::async_trait;
-use bytes::Bytes;
+use bytes::Buf;
 use futures::FutureExt;
 use futures::TryFutureExt;
 use log::debug;
@@ -56,7 +53,7 @@ use crate::*;
 ///
 /// # Examples
 ///
-/// ```
+/// ```no_run
 /// use anyhow::Result;
 /// use opendal::layers::LoggingLayer;
 /// use opendal::services;
@@ -150,10 +147,10 @@ impl LoggingLayer {
     }
 }
 
-impl<A: Accessor> Layer<A> for LoggingLayer {
-    type LayeredAccessor = LoggingAccessor<A>;
+impl<A: Access> Layer<A> for LoggingLayer {
+    type LayeredAccess = LoggingAccessor<A>;
 
-    fn layer(&self, inner: A) -> Self::LayeredAccessor {
+    fn layer(&self, inner: A) -> Self::LayeredAccess {
         let meta = inner.info();
         LoggingAccessor {
             inner,
@@ -203,7 +200,7 @@ impl LoggingContext {
 }
 
 #[derive(Clone, Debug)]
-pub struct LoggingAccessor<A: Accessor> {
+pub struct LoggingAccessor<A: Access> {
     inner: A,
 
     ctx: LoggingContext,
@@ -211,9 +208,7 @@ pub struct LoggingAccessor<A: Accessor> {
 
 static LOGGING_TARGET: &str = "opendal::services";
 
-#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
-#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
-impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
+impl<A: Access> LayeredAccess for LoggingAccessor<A> {
     type Inner = A;
     type Reader = LoggingReader<A::Reader>;
     type BlockingReader = LoggingReader<A::BlockingReader>;
@@ -286,26 +281,22 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
         debug!(
             target: LOGGING_TARGET,
-            "service={} operation={} path={} range={} -> started",
+            "service={} operation={} path={} -> started",
             self.ctx.scheme,
             Operation::Read,
             path,
-            args.range()
         );
 
-        let range = args.range();
-
         self.inner
             .read(path, args)
             .await
             .map(|(rp, r)| {
                 debug!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} range={} -> got reader",
+                    "service={} operation={} path={} -> got reader",
                     self.ctx.scheme,
                     Operation::Read,
                     path,
-                    range
                 );
                 (
                     rp,
@@ -317,11 +308,10 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
                     log!(
                         target: LOGGING_TARGET,
                         lvl,
-                        "service={} operation={} path={} range={} -> {}",
+                        "service={} operation={} path={} -> {}",
                         self.ctx.scheme,
                         Operation::Read,
                         path,
-                        range,
                         self.ctx.error_print(&err)
                     )
                 }
@@ -685,11 +675,10 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
     fn blocking_read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::BlockingReader)> {
         debug!(
             target: LOGGING_TARGET,
-            "service={} operation={} path={} range={} -> started",
+            "service={} operation={} path={} -> started",
             self.ctx.scheme,
             Operation::BlockingRead,
             path,
-            args.range(),
         );
 
         self.inner
@@ -697,11 +686,10 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
             .map(|(rp, r)| {
                 debug!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} range={} -> got reader",
+                    "service={} operation={} path={} -> got reader",
                     self.ctx.scheme,
                     Operation::BlockingRead,
                     path,
-                    args.range(),
                 );
                 let r = LoggingReader::new(self.ctx.clone(), Operation::BlockingRead, path, r);
                 (rp, r)
@@ -711,11 +699,10 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
                     log!(
                         target: LOGGING_TARGET,
                         lvl,
-                        "service={} operation={} path={} range={} -> {}",
+                        "service={} operation={} path={} -> {}",
                         self.ctx.scheme,
                         Operation::BlockingRead,
                         path,
-                        args.range(),
                         self.ctx.error_print(&err)
                     );
                 }
@@ -960,7 +947,7 @@ pub struct LoggingReader<R> {
     path: String,
     op: Operation,
 
-    read: u64,
+    read: AtomicU64,
     inner: R,
 }
 
@@ -971,7 +958,7 @@ impl<R> LoggingReader<R> {
             op,
             path: path.to_string(),
 
-            read: 0,
+            read: AtomicU64::new(0),
             inner: reader,
         }
     }
@@ -985,190 +972,74 @@ impl<R> Drop for LoggingReader<R> {
             self.ctx.scheme,
             self.op,
             self.path,
-            self.read
+            self.read.load(Ordering::Relaxed)
         );
     }
 }
 
 impl<R: oio::Read> oio::Read for LoggingReader<R> {
-    fn poll_read(&mut self, cx: &mut Context<'_>, buf: &mut [u8]) -> Poll<Result<usize>> {
-        let buf_size = buf.len();
-
-        match self.inner.poll_read(cx, buf) {
-            Poll::Ready(res) => match res {
-                Ok(n) => {
-                    self.read += n as u64;
-                    trace!(
-                        target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> buf size: {}B, read {}B ",
-                        self.ctx.scheme,
-                        ReadOperation::Read,
-                        self.path,
-                        self.read,
-                        buf_size,
-                        n
-                    );
-                    Poll::Ready(Ok(n))
-                }
-                Err(err) => {
-                    if let Some(lvl) = self.ctx.error_level(&err) {
-                        log!(
-                            target: LOGGING_TARGET,
-                            lvl,
-                            "service={} operation={} path={} read={} -> read failed: {}",
-                            self.ctx.scheme,
-                            ReadOperation::Read,
-                            self.path,
-                            self.read,
-                            self.ctx.error_print(&err)
-                        )
-                    }
-                    Poll::Ready(Err(err))
-                }
-            },
-            Poll::Pending => {
+    async fn read_at(&self, offset: u64, limit: usize) -> Result<Buffer> {
+        match self.inner.read_at(offset, limit).await {
+            Ok(bs) => {
+                self.read
+                    .fetch_add(bs.remaining() as u64, Ordering::Relaxed);
                 trace!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> buf size: {}B, read pending",
+                    "service={} operation={} path={} read={} -> read returns {}B",
                     self.ctx.scheme,
                     ReadOperation::Read,
                     self.path,
-                    self.read,
-                    buf_size
+                    self.read.load(Ordering::Relaxed),
+                    bs.remaining()
                 );
-                Poll::Pending
+                Ok(bs)
             }
-        }
-    }
-
-    fn poll_seek(&mut self, cx: &mut Context<'_>, pos: io::SeekFrom) -> Poll<Result<u64>> {
-        match self.inner.poll_seek(cx, pos) {
-            Poll::Ready(res) => match res {
-                Ok(n) => {
-                    trace!(
+            Err(err) => {
+                if let Some(lvl) = self.ctx.error_level(&err) {
+                    log!(
                         target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> seek to {pos:?}, current offset {n}",
+                        lvl,
+                        "service={} operation={} path={} read={} -> read failed: {}",
                         self.ctx.scheme,
-                        ReadOperation::Seek,
+                        ReadOperation::Read,
                         self.path,
-                        self.read,
-                    );
-                    Poll::Ready(Ok(n))
+                        self.read.load(Ordering::Relaxed),
+                        self.ctx.error_print(&err),
+                    )
                 }
-                Err(err) => {
-                    if let Some(lvl) = self.ctx.error_level(&err) {
-                        log!(
-                            target: LOGGING_TARGET,
-                            lvl,
-                            "service={} operation={} path={} read={} -> seek to {pos:?} failed: {}",
-                            self.ctx.scheme,
-                            ReadOperation::Seek,
-                            self.path,
-                            self.read,
-                            self.ctx.error_print(&err),
-                        )
-                    }
-                    Poll::Ready(Err(err))
-                }
-            },
-            Poll::Pending => {
-                trace!(
-                    target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> seek to {pos:?} pending",
-                    self.ctx.scheme,
-                    ReadOperation::Seek,
-                    self.path,
-                    self.read
-                );
-                Poll::Pending
-            }
-        }
-    }
-
-    fn poll_next(&mut self, cx: &mut Context<'_>) -> Poll<Option<Result<Bytes>>> {
-        match self.inner.poll_next(cx) {
-            Poll::Ready(res) => match res {
-                Some(Ok(bs)) => {
-                    self.read += bs.len() as u64;
-                    trace!(
-                        target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> next returns {}B",
-                        self.ctx.scheme,
-                        ReadOperation::Next,
-                        self.path,
-                        self.read,
-                        bs.len()
-                    );
-                    Poll::Ready(Some(Ok(bs)))
-                }
-                Some(Err(err)) => {
-                    if let Some(lvl) = self.ctx.error_level(&err) {
-                        log!(
-                            target: LOGGING_TARGET,
-                            lvl,
-                            "service={} operation={} path={} read={} -> next failed: {}",
-                            self.ctx.scheme,
-                            ReadOperation::Next,
-                            self.path,
-                            self.read,
-                            self.ctx.error_print(&err),
-                        )
-                    }
-                    Poll::Ready(Some(Err(err)))
-                }
-                None => {
-                    trace!(
-                        target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> next returns None",
-                        self.ctx.scheme,
-                        ReadOperation::Next,
-                        self.path,
-                        self.read,
-                    );
-                    Poll::Ready(None)
-                }
-            },
-            Poll::Pending => {
-                trace!(
-                    target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> next returns pending",
-                    self.ctx.scheme,
-                    ReadOperation::Next,
-                    self.path,
-                    self.read
-                );
-                Poll::Pending
+                Err(err)
             }
         }
     }
 }
 
 impl<R: oio::BlockingRead> oio::BlockingRead for LoggingReader<R> {
-    fn read(&mut self, buf: &mut [u8]) -> Result<usize> {
-        match self.inner.read(buf) {
-            Ok(n) => {
-                self.read += n as u64;
+    fn read_at(&self, offset: u64, limit: usize) -> Result<Buffer> {
+        match self.inner.read_at(offset, limit) {
+            Ok(bs) => {
+                self.read
+                    .fetch_add(bs.remaining() as u64, Ordering::Relaxed);
                 trace!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> data read {}B",
+                    "service={} operation={} path={} read={} -> read returns {}B",
                     self.ctx.scheme,
                     ReadOperation::BlockingRead,
                     self.path,
-                    self.read,
-                    n
+                    self.read.load(Ordering::Relaxed),
+                    bs.remaining()
                 );
-                Ok(n)
+                Ok(bs)
             }
             Err(err) => {
                 if let Some(lvl) = self.ctx.error_level(&err) {
                     log!(
                         target: LOGGING_TARGET,
                         lvl,
-                        "service={} operation={} path={} read={} -> data read failed: {}",
+                        "service={} operation={} path={} read={} -> read failed: {}",
                         self.ctx.scheme,
                         ReadOperation::BlockingRead,
                         self.path,
-                        self.read,
+                        self.read.load(Ordering::Relaxed),
                         self.ctx.error_print(&err),
                     );
                 }
@@ -1176,72 +1047,6 @@ impl<R: oio::BlockingRead> oio::BlockingRead for LoggingReader<R> {
             }
         }
     }
-
-    #[inline]
-    fn seek(&mut self, pos: io::SeekFrom) -> Result<u64> {
-        match self.inner.seek(pos) {
-            Ok(n) => {
-                trace!(
-                    target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> data seek to offset {n}",
-                    self.ctx.scheme,
-                    ReadOperation::BlockingSeek,
-                    self.path,
-                    self.read,
-                );
-                Ok(n)
-            }
-            Err(err) => {
-                if let Some(lvl) = self.ctx.error_level(&err) {
-                    log!(
-                        target: LOGGING_TARGET,
-                        lvl,
-                        "service={} operation={} path={} read={} -> data read failed: {}",
-                        self.ctx.scheme,
-                        ReadOperation::BlockingSeek,
-                        self.path,
-                        self.read,
-                        self.ctx.error_print(&err),
-                    );
-                }
-                Err(err)
-            }
-        }
-    }
-
-    fn next(&mut self) -> Option<Result<Bytes>> {
-        match self.inner.next() {
-            Some(Ok(bs)) => {
-                self.read += bs.len() as u64;
-                trace!(
-                    target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> data read {}B",
-                    self.ctx.scheme,
-                    ReadOperation::BlockingNext,
-                    self.path,
-                    self.read,
-                    bs.len()
-                );
-                Some(Ok(bs))
-            }
-            Some(Err(err)) => {
-                if let Some(lvl) = self.ctx.error_level(&err) {
-                    log!(
-                        target: LOGGING_TARGET,
-                        lvl,
-                        "service={} operation={} path={} read={} -> data read failed: {}",
-                        self.ctx.scheme,
-                        ReadOperation::BlockingNext,
-                        self.path,
-                        self.read,
-                        self.ctx.error_print(&err),
-                    )
-                }
-                Some(Err(err))
-            }
-            None => None,
-        }
-    }
 }
 
 pub struct LoggingWriter<W> {
@@ -1267,8 +1072,8 @@ impl<W> LoggingWriter<W> {
 }
 
 impl<W: oio::Write> oio::Write for LoggingWriter<W> {
-    fn poll_write(&mut self, cx: &mut Context<'_>, bs: &dyn oio::WriteBuf) -> Poll<Result<usize>> {
-        match ready!(self.inner.poll_write(cx, bs)) {
+    async fn write(&mut self, bs: Buffer) -> Result<usize> {
+        match self.inner.write(bs.clone()).await {
             Ok(n) => {
                 self.written += n as u64;
                 trace!(
@@ -1278,10 +1083,10 @@ impl<W: oio::Write> oio::Write for LoggingWriter<W> {
                     WriteOperation::Write,
                     self.path,
                     self.written,
-                    bs.remaining(),
+                    bs.len(),
                     n,
                 );
-                Poll::Ready(Ok(n))
+                Ok(n)
             }
             Err(err) => {
                 if let Some(lvl) = self.ctx.error_level(&err) {
@@ -1296,13 +1101,13 @@ impl<W: oio::Write> oio::Write for LoggingWriter<W> {
                         self.ctx.error_print(&err),
                     )
                 }
-                Poll::Ready(Err(err))
+                Err(err)
             }
         }
     }
 
-    fn poll_abort(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
-        match ready!(self.inner.poll_abort(cx)) {
+    async fn abort(&mut self) -> Result<()> {
+        match self.inner.abort().await {
             Ok(_) => {
                 trace!(
                     target: LOGGING_TARGET,
@@ -1312,7 +1117,7 @@ impl<W: oio::Write> oio::Write for LoggingWriter<W> {
                     self.path,
                     self.written,
                 );
-                Poll::Ready(Ok(()))
+                Ok(())
             }
             Err(err) => {
                 if let Some(lvl) = self.ctx.error_level(&err) {
@@ -1327,13 +1132,13 @@ impl<W: oio::Write> oio::Write for LoggingWriter<W> {
                         self.ctx.error_print(&err),
                     )
                 }
-                Poll::Ready(Err(err))
+                Err(err)
             }
         }
     }
 
-    fn poll_close(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
-        match ready!(self.inner.poll_close(cx)) {
+    async fn close(&mut self) -> Result<()> {
+        match self.inner.close().await {
             Ok(_) => {
                 debug!(
                     target: LOGGING_TARGET,
@@ -1343,7 +1148,7 @@ impl<W: oio::Write> oio::Write for LoggingWriter<W> {
                     self.path,
                     self.written
                 );
-                Poll::Ready(Ok(()))
+                Ok(())
             }
             Err(err) => {
                 if let Some(lvl) = self.ctx.error_level(&err) {
@@ -1358,15 +1163,15 @@ impl<W: oio::Write> oio::Write for LoggingWriter<W> {
                         self.ctx.error_print(&err),
                     )
                 }
-                Poll::Ready(Err(err))
+                Err(err)
             }
         }
     }
 }
 
 impl<W: oio::BlockingWrite> oio::BlockingWrite for LoggingWriter<W> {
-    fn write(&mut self, bs: &dyn oio::WriteBuf) -> Result<usize> {
-        match self.inner.write(bs) {
+    fn write(&mut self, bs: Buffer) -> Result<usize> {
+        match self.inner.write(bs.clone()) {
             Ok(n) => {
                 self.written += n as u64;
                 trace!(
@@ -1376,7 +1181,7 @@ impl<W: oio::BlockingWrite> oio::BlockingWrite for LoggingWriter<W> {
                     WriteOperation::BlockingWrite,
                     self.path,
                     self.written,
-                    bs.remaining(),
+                    bs.len(),
                     n
                 );
                 Ok(n)
@@ -1474,11 +1279,9 @@ impl<P> Drop for LoggingLister<P> {
     }
 }
 
-#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
-#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
 impl<P: oio::List> oio::List for LoggingLister<P> {
-    fn poll_next(&mut self, cx: &mut Context<'_>) -> Poll<Result<Option<oio::Entry>>> {
-        let res = ready!(self.inner.poll_next(cx));
+    async fn next(&mut self) -> Result<Option<oio::Entry>> {
+        let res = self.inner.next().await;
 
         match &res {
             Ok(Some(de)) => {
@@ -1516,7 +1319,7 @@ impl<P: oio::List> oio::List for LoggingLister<P> {
             }
         };
 
-        Poll::Ready(res)
+        res
     }
 }
 
