@@ -20,7 +20,7 @@ use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::sync::Arc;
 
-use async_trait::async_trait;
+use bytes::Buf;
 use http::StatusCode;
 use log::debug;
 use reqsign::GoogleCredentialLoader;
@@ -33,6 +33,7 @@ use serde_json;
 use super::core::*;
 use super::error::parse_error;
 use super::lister::GcsLister;
+use super::reader::GcsReader;
 use super::writer::GcsWriter;
 use super::writer::GcsWriters;
 use crate::raw::*;
@@ -57,9 +58,9 @@ pub struct GcsConfig {
     scope: Option<String>,
     /// Service Account for gcs.
     service_account: Option<String>,
-    /// credential string for GCS service
+    /// Credentials string for GCS service OAuth2 authentication.
     credential: Option<String>,
-    /// credential path for GCS service.
+    /// Local path to credentials file for GCS service OAuth2 authentication.
     credential_path: Option<String>,
     /// The predefined acl for GCS.
     predefined_acl: Option<String>,
@@ -150,7 +151,13 @@ impl GcsBuilder {
         self
     }
 
-    /// set the base64 hashed credentials string used for OAuth2
+    /// set the base64 hashed credentials string used for OAuth2 authentication.
+    ///
+    /// this method allows to specify the credentials directly as a base64 hashed string.
+    /// alternatively, you can use `credential_path()` to provide the local path to a credentials file.
+    /// we will use one of `credential` and `credential_path` to complete the OAuth2 authentication.
+    ///
+    /// Reference: [Google Cloud Storage Authentication](https://cloud.google.com/docs/authentication).
     pub fn credential(&mut self, credential: &str) -> &mut Self {
         if !credential.is_empty() {
             self.config.credential = Some(credential.to_string())
@@ -158,7 +165,12 @@ impl GcsBuilder {
         self
     }
 
-    /// set the credentials path of GCS.
+    /// set the local path to credentials file which is used for OAuth2 authentication.
+    ///
+    /// credentials file contains the original credentials that have not been base64 hashed.
+    /// we will use one of `credential` and `credential_path` to complete the OAuth2 authentication.
+    ///
+    /// Reference: [Google Cloud Storage Authentication](https://cloud.google.com/docs/authentication).
     pub fn credential_path(&mut self, path: &str) -> &mut Self {
         if !path.is_empty() {
             self.config.credential_path = Some(path.to_string())
@@ -318,10 +330,8 @@ pub struct GcsBackend {
     core: Arc<GcsCore>,
 }
 
-#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
-#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
-impl Accessor for GcsBackend {
-    type Reader = IncomingAsyncBody;
+impl Access for GcsBackend {
+    type Reader = GcsReader;
     type Writer = GcsWriters;
     type Lister = oio::PageLister<GcsLister>;
     type BlockingReader = ();
@@ -339,8 +349,7 @@ impl Accessor for GcsBackend {
                 stat_with_if_none_match: true,
 
                 read: true,
-                read_can_next: true,
-                read_with_range: true,
+
                 read_with_if_match: true,
                 read_with_if_none_match: true,
 
@@ -382,10 +391,10 @@ impl Accessor for GcsBackend {
             return Err(parse_error(resp).await?);
         }
 
-        let slc = resp.into_body().bytes().await?;
+        let slc = resp.into_body();
 
         let meta: GetObjectJsonResponse =
-            serde_json::from_slice(&slc).map_err(new_json_deserialize_error)?;
+            serde_json::from_reader(slc.reader()).map_err(new_json_deserialize_error)?;
 
         let mut m = Metadata::new(EntryMode::FILE);
 
@@ -407,16 +416,10 @@ impl Accessor for GcsBackend {
     }
 
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        let resp = self.core.gcs_get_object(path, &args).await?;
-
-        if resp.status().is_success() {
-            let size = parse_content_length(resp.headers())?;
-            Ok((RpRead::new().with_size(size), resp.into_body()))
-        } else if resp.status() == StatusCode::RANGE_NOT_SATISFIABLE {
-            Ok((RpRead::new(), IncomingAsyncBody::empty()))
-        } else {
-            Err(parse_error(resp).await?)
-        }
+        Ok((
+            RpRead::default(),
+            GcsReader::new(self.core.clone(), path, args),
+        ))
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
@@ -454,7 +457,6 @@ impl Accessor for GcsBackend {
         let resp = self.core.gcs_copy_object(from, to).await?;
 
         if resp.status().is_success() {
-            resp.into_body().consume().await?;
             Ok(RpCopy::default())
         } else {
             Err(parse_error(resp).await?)
@@ -468,7 +470,7 @@ impl Accessor for GcsBackend {
             PresignOperation::Read(v) => self.core.gcs_get_object_xml_request(path, v)?,
             PresignOperation::Write(v) => {
                 self.core
-                    .gcs_insert_object_xml_request(path, v, AsyncBody::Empty)?
+                    .gcs_insert_object_xml_request(path, v, Buffer::new())?
             }
         };
 
@@ -517,7 +519,7 @@ impl Accessor for GcsBackend {
                 .trim_matches('"');
             let multipart: Multipart<MixedPart> = Multipart::new()
                 .with_boundary(boundary)
-                .parse(resp.into_body().bytes().await?)?;
+                .parse(resp.into_body().to_bytes())?;
             let parts = multipart.into_parts();
 
             let mut batched_result = Vec::with_capacity(parts.len());
